/***************************************************************************/
/*                                                                         */
/* (c) Copyright IBM Corp. 2001  All rights reserved.                      */
/*                                                                         */
/* This sample program is owned by International Business Machines         */
/* Corporation or one of its subsidiaries ("IBM") and is copyrighted       */
/* and licensed, not sold.                                                 */
/*                                                                         */
/* You may copy, modify, and distribute this sample program in any         */
/* form without payment to IBM,  for any purpose including developing,     */
/* using, marketing or distributing programs that include or are           */
/* derivative works of the sample program.                                 */
/*                                                                         */
/* The sample program is provided to you on an "AS IS" basis, without      */
/* warranty of any kind.  IBM HEREBY  EXPRESSLY DISCLAIMS ALL WARRANTIES,  */
/* EITHER EXPRESS OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED   */
/* WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.     */
/* Some jurisdictions do not allow for the exclusion or limitation of      */
/* implied warranties, so the above limitations or exclusions may not      */
/* apply to you.  IBM shall not be liable for any damages you suffer as    */
/* a result of using, modifying or distributing the sample program or      */
/* its derivatives.                                                        */
/*                                                                         */
/***************************************************************************/
/*                                                                         */
/* Program name: mqsyncjms                                                 */
/*                                                                         */
/* Description: Sample java program that shows jms syncpointing of         */
/*              messages during put requests                               */
/*                                                                         */
/***************************************************************************/
/*                                                                         */
/* Function:                                                               */
/*                                                                         */
/*  This program is a variation of the mqsync.c program that can be found  */
/*  on the MQSeries sample code section of the web at:                     */
/*                                                                         */
/*    http://www.developer.ibm.com/sdp/library/sample_code.html            */
/*                                                                         */
/*  This program is written in java and shows the use of the jms objects   */
/*  and methods to put messages in syncpoint by checking the message       */
/*  buffer for the following text:                                         */
/*                                                                         */
/*      SYNC or sync: the message is to be put in syncpoint                */
/*      CMIT or cmit: the syncpointed messages are to be committed         */
/*      BACK or back: the syncpointed messages are to be backed out        */
/*                                                                         */
/* The text must be in all upper case or all lower case and can be found   */
/* anywhere in the message text.                                           */
/*                                                                         */
/* If both backout and commit are specified, the backout will be done.     */
/* The best way to run this program is in conjunction with the amqsget     */
/* sample program.  The mqsyncjms program can be run in one window while   */
/* the amqsget program can be run in another.  The mqsyncjms program can   */
/* be run as follows:                                                      */
/*                                                                         */
/*     java mqsyncjms connectionFactoryName queue                          */
/*     message 1                                                           */
/*     message 2 sync                                                      */
/*     message 3                                                           */
/*     message 4 sync                                                      */
/*     message 5 cmit                                                      */
/*     message 6 sync                                                      */
/*     message 7                                                           */
/*     message 8 back                                                      */
/*     message 9                                                           */
/*                                                                         */
/* The amqsget program would then run as follows:                          */
/*                                                                         */
/*     amqsget SYSTEM.DEFAULT.LOCAL.QUEUE                                  */
/*     message 1                                                           */
/*     message 3                                                           */
/*     message 5 cmit                 shows up here because not in sync    */
/*     message 2 SYNC                                                      */
/*     message 4 sync                                                      */
/*     message 7                                                           */
/*     message 8 back                 this caused message 6 to be disarded */
/*     message 9                                                           */
/*                                                                         */
/* This program has been tested with MQSeries V5.1 and JDK 1.1.8.          */
/*                                                                         */
/***************************************************************************/
/*                                                                         */
/* The connectionFactoryName and queue parameters to this program need to  */
/* be setup so that they can be found.  This is done via the JMS Admin-    */
/* istration tool that is supplied with MQSeries Java (SupportPac MA88)    */
/* and is described in the Using Java manual.  This program was tested     */
/*  using a file system context set up in a jmsadmin.config as folllows:   */
/*                                                                         */
/*    INITIAL_CONTEXT_FACTORY=com.sun.jndi.fscontext.RefFSContextFactory   */
/*    PROVIDER_URL=file:e://support//jms//sample                           */
/*    SECURITY_AUTHENTICATION=none                                         */
/*                                                                         */
/* The actual JMSAdmin utility was run with the following definitions:     */
/*                                                                         */
/*    def qcf(sampleQCF) qmanager(bower1.qmgr)                             */
/*    def q(sampleQ) qu(SYSTEM.DEFAULT.LOCAL.QUEUE) TC(MQ)                 */
/*                                                                         */
/* The actual program execution:                                           */
/*                                                                         */
/*    java mqsyncjms sampleQCF sampleQ                                     */
/*                                                                         */
/***************************************************************************/
import java.io.*;

import javax.jms.*;
import javax.naming.*;
import javax.naming.directory.*;

public class mqsyncjms {

    QueueConnectionFactory connectionFactory;
    QueueConnection connection;
    QueueSession session;
    QueueSession sessionSP;
    QueueSender sender;
    QueueSender senderSP;
    Queue        ioQueue;     
    TextMessage message;
    Context     ctx = null;
    boolean     bUOW;

    public mqsyncjms(String[] param) {
        if (param.length < 2) {
           System.err.println("Usage: java mqsyncjms <connectionFactoryName> <queue>");
           System.exit(1);
        } else {
           System.out.println("mqsyncjms sample program");
           System.out.println("  using connection factory = " + param[0]);
           System.out.println("  using queue              = " + param[1]);
        }

        try {
           /*************************************************************/
           /* This program is hard-coded to use a file system context.  */
           /* If you wanted to use something else, you could pass it in */
           /* as a parameter and put it in the hash table below.        */
           /*************************************************************/
           java.util.Hashtable env = new java.util.Hashtable();
           env.put(Context.INITIAL_CONTEXT_FACTORY,
                   "com.sun.jndi.fscontext.RefFSContextFactory");
           env.put(Context.PROVIDER_URL, "file:e://support//jms//sample");

           ctx = new InitialContext(env);  
           connectionFactory = (QueueConnectionFactory)ctx.lookup(param[0]);
        } catch(NamingException e) {
           System.err.println("Error during lookup of Connection Factory " +
                              e.getMessage());
           System.exit(1);
        }

        try {
           connection = connectionFactory.createQueueConnection();
           /*********************************************************/
           /* This program creates two sessions/senders.  One pair  */
           /* is used to put messages in a unit of work.  The other */
           /* pair is used to put the messages out immediately.     */
           /*********************************************************/

           session   = connection.createQueueSession(true, 
                                      Session.AUTO_ACKNOWLEDGE);
           sessionSP = connection.createQueueSession(true, 
                                      Session.CLIENT_ACKNOWLEDGE);

           ioQueue = (Queue)ctx.lookup(param[1]);       
          
           sender   = session.createSender(ioQueue);  
           senderSP = sessionSP.createSender(ioQueue);  

           InputStreamReader isr = new InputStreamReader(System.in);
           BufferedReader    br  = new BufferedReader(isr);
           String sInput;

           System.out.println("Use SYNC or sync in message text for syncpoint'ed message");
           System.out.println("Use CMIT or cmit in message text to commit messages");
           System.out.println("Use BACK or back in message text to backout (discard) messages");

           /****************************************************/
           /* As long as the user doesn't enter an empty line, */
           /* we'll put messages on the queue.  When directed  */
           /* to commit or roll back the messages, the program */
           /* will either perform a commit or a rollback.      */
           /****************************************************/
           do {
              sInput = br.readLine();
              if (sInput.length() > 0) {
                
                 /*********************************************************/
                 /* Check to see if the message is part of a unit of work */
                 /* or if it can be put immediately.                      */
                 /*********************************************************/
                 if (    (sInput.indexOf("SYNC") >= 0)
                      || (sInput.indexOf("sync") >= 0) ) {
                    bUOW = true; /* at least one message in unit of work */
                    message = sessionSP.createTextMessage();
                    message.setText(sInput);
                    senderSP.send(message); 
                 } else {
                    message = session.createTextMessage();
                    message.setText(sInput);
                    sender.send(message); 
                    session.commit();
                 }
                 
                 /*************************************************/
                 /* See if the message indicates to commit or     */
                 /* rollback the messages in the unit of work     */
                 /*************************************************/
                 if (    (sInput.indexOf("BACK") >= 0)
                      || (sInput.indexOf("back") >= 0) ) {
                    sessionSP.rollback();
                    bUOW = false; /* UOW now empty */
                 } else if (   (sInput.indexOf("CMIT") >= 0) 
                            || (sInput.indexOf("cmit") >= 0) ) {
                    sessionSP.commit();
                    bUOW = false; /* UOW now empty */
                 }
              }
           } while (sInput.length() > 0);

           /***************************************************/
           /* At least one message was left in the UOW, this  */
           /* program will roll back any uncomitted messages. */
           /***************************************************/
           if (bUOW) {
              System.out.println("No commit issued, message being rolled back");
              sessionSP.rollback();
           }

           /**********************************/
           /* Close down all of the objects. */
           /**********************************/
           senderSP.close();
           sessionSP.close();
           sender.close();
           session.close();
           connection.close();
           ctx.close();
        } catch (Exception e) {
           System.err.println(e.getMessage());
           if (e instanceof JMSException) {
              Exception le = ((JMSException)e).getLinkedException();
              if (le != null) {
                 System.err.println(le);
              }
           }
           System.exit(1);
        }
    }

    public static void main (String args[]) {
        mqsyncjms app = new mqsyncjms(args);
    }

}
